TYPE_BOT = "bot"
TYPE_USER = "user"
TYPE_SYSTEM = "system"

DATE_PATTERN = "%Y%m%d"
TIMEOUT_SECONDS = 30

PROVIDER_OPENAI = "openai"
PROVIDER_MINIMAX = "minimax"
PROVIDER_GOOGLE = "google"
PROVIDER_XUNFEI = "spark"
PROVIDER_CLAUDE = "claude"
PROVIDER_INTERNLM = "internlm"

DEFAULT_TIMEOUT_SECONDS = 600

